class SubscriberNotFoundException(Exception):
    def __init__(self, subscriber_id: str):
        self.subscriber_id = subscriber_id
        super().__init__(
            f"Subscriber with id {subscriber_id} has not been found in the container and cannot be registered"
        )



from collections import defaultdict
from typing import List, DefaultDict

from buz.event import Subscriber
from buz.event.sync import (
    SubscriberAlreadyRegisteredException,
    SubscriberNotRegisteredException,
)
from buz.event.sync import Locator


class InstanceLocator(Locator):
    def __init__(self) -> None:
        self.__mapping: DefaultDict[str, List[Subscriber]] = defaultdict(lambda: [])

    def register(self, subscriber: Subscriber) -> None:
        event_fqn = subscriber.event_subscribed_to()
        self.__guard_subscriber_already_registered(event_fqn, subscriber)
        self.__mapping[event_fqn].append(subscriber)

    def __guard_subscriber_already_registered(self, event_fqn: str, subscriber: Subscriber) -> None:
        if subscriber in self.__mapping[event_fqn]:
            raise SubscriberAlreadyRegisteredException(subscriber)

    def unregister(self, subscriber: Subscriber) -> None:
        event_fqn = subscriber.event_subscribed_to()
        self.__guard_subscriber_not_registered(event_fqn, subscriber)
        self.__mapping[event_fqn].remove(subscriber)

    def __guard_subscriber_not_registered(self, event_fqn: str, subscriber: Subscriber) -> None:
        if subscriber not in self.__mapping[event_fqn]:
            raise SubscriberNotRegisteredException(subscriber)

    def get(self, event_fqn: str) -> List[Subscriber]:
        return self.__mapping.get(event_fqn, [])



from buz.event import Subscriber


class SubscriberNotRegisteredException(Exception):
    def __init__(self, subscriber: Subscriber):
        self.subscriber = subscriber
        super().__init__("Subscriber has not been registered and cannot be unregistered")



from collections import defaultdict
from typing import List, DefaultDict

from pypendency.builder import Container

from buz.event import Subscriber
from buz.event.pypendency import (
    SubscriberAlreadyRegisteredException,
    SubscriberNotRegisteredException,
    SubscriberNotFoundException,
)
from buz.event.sync.locator import Locator


class ContainerLocator(Locator):
    CHECK_MODE_REGISTER_TIME = "register"
    CHECK_MODE_GET_TIME = "get"

    def __init__(self, container: Container, check_mode: str = CHECK_MODE_REGISTER_TIME) -> None:
        self.__container = container
        self.__check_mode = check_mode
        self.__mapping: DefaultDict[str, List[str]] = defaultdict(lambda: [])
        self.__subscriber_ids: List[str] = []
        self.__subscribers_resolved = False

    def register(self, subscriber_id: str) -> None:
        self.__guard_subscriber_already_registered(subscriber_id)
        if self.__check_mode == self.CHECK_MODE_REGISTER_TIME:
            self.__guard_subscriber_not_found(subscriber_id)
        self.__subscriber_ids.append(subscriber_id)
        self.__subscribers_resolved = False

    def __guard_subscriber_already_registered(self, subscriber_id: str) -> None:
        if subscriber_id in self.__subscriber_ids:
            raise SubscriberAlreadyRegisteredException(subscriber_id)

    def __guard_subscriber_not_found(self, subscriber_id: str) -> None:
        if not self.__container.has(subscriber_id):
            raise SubscriberNotFoundException(subscriber_id)

    def unregister(self, subscriber_id: str) -> None:
        self.__guard_subscriber_not_registered(subscriber_id)
        self.__subscriber_ids.remove(subscriber_id)
        self.__subscribers_resolved = False

    def __guard_subscriber_not_registered(self, subscriber_id: str) -> None:
        if subscriber_id not in self.__subscriber_ids:
            raise SubscriberNotRegisteredException(subscriber_id)

    def get(self, event_fqn: str) -> List[Subscriber]:
        if not self.__subscribers_resolved:
            self.__resolve_subscribers()
        subscriber_ids = self.__mapping.get(event_fqn, [])
        return [self.__container.get(subscriber_id) for subscriber_id in subscriber_ids]

    def __resolve_subscribers(self) -> None:
        for subscriber_id in self.__subscriber_ids:
            if self.__check_mode == self.CHECK_MODE_GET_TIME:
                self.__guard_subscriber_not_found(subscriber_id)
            subscriber = self.__container.get(subscriber_id)
            event_fqn = subscriber.event_subscribed_to()
            self.__mapping[event_fqn].append(subscriber_id)



class SubscriberAlreadyRegisteredException(Exception):
    def __init__(self, subscriber_id: str):
        self.subscriber_id = subscriber_id
        super().__init__(
            f"Subscriber with id {subscriber_id} has been registered and cannot be registered another time"
        )



from buz.event.sync.subscriber_already_registered_exception import SubscriberAlreadyRegisteredException
from buz.event.sync.subscriber_not_registered_exception import SubscriberNotRegisteredException
from buz.event.sync.locator import Locator
from buz.event.sync.instance_locator import InstanceLocator
from buz.event.sync.sync_event_bus import SyncEventBus


__all__ = [
    "Locator",
    "SubscriberAlreadyRegisteredException",
    "SubscriberNotRegisteredException",
    "InstanceLocator",
    "SyncEventBus",
]



from buz.event.pypendency.subscriber_already_registered_exception import SubscriberAlreadyRegisteredException
from buz.event.pypendency.subscriber_not_found_exception import SubscriberNotFoundException
from buz.event.pypendency.subscriber_not_registered_exception import SubscriberNotRegisteredException
from buz.event.pypendency.container_locator import ContainerLocator

__all__ = [
    "SubscriberAlreadyRegisteredException",
    "SubscriberNotRegisteredException",
    "SubscriberNotFoundException",
    "ContainerLocator",
]



from buz.event import Subscriber


class SubscriberAlreadyRegisteredException(Exception):
    def __init__(self, subscriber: Subscriber):
        self.subscriber = subscriber
        super().__init__("Subscriber has been registered and cannot be registered another time")



from abc import ABC, abstractmethod

from buz.event import Event


class Subscriber(ABC):
    @staticmethod
    @abstractmethod
    def event_subscribed_to() -> str:
        pass

    @abstractmethod
    def consume(self, event: Event) -> None:
        pass



from setuptools import setup, find_packages

with open("README.md", "r") as readme:
    README = readme.read()

setup(
    name="buz",
    version="0.3.0",
    packages=find_packages("src"),
    package_dir={"": "src"},
    author="Luis Pintado Lozano",
    author_email="luis.pintado.lozano@gmail.com",
    description="Light, simple and extensible implementations of event, command and query buses",
    long_description=README,
    long_description_content_type="text/markdown",
    license="MIT License",
    url="https://github.com/Feverup/buz",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries",
        "Typing :: Typed",
    ],
    python_requires=">=3.6",
    install_requires=['dataclasses;python_version<"3.7.0"'],
    extras_require={"pypendency": ["pypendency~=0.1.0"]},
    include_package_data=True,
)



from abc import ABC
from dataclasses import dataclass, field
from datetime import datetime
from uuid import uuid4


@dataclass(frozen=True)  # type: ignore
class Event(ABC):
    id: str = field(init=False, default_factory=lambda: str(uuid4()))
    created_at: str = field(init=False, default_factory=lambda: str(datetime.now()))

    @classmethod
    def fqn(cls) -> str:
        return f"{cls.__module__}.{cls.__name__}"



from typing import Collection

from buz.event import Event, EventBus
from buz.event.sync import Locator


class SyncEventBus(EventBus):
    def __init__(self, locator: Locator):
        self.locator = locator

    def publish(self, event: Event) -> None:
        subscribers = self.locator.get(event.fqn())
        for subscriber in subscribers:
            subscriber.consume(event)

    def bulk_publish(self, events: Collection[Event]) -> None:
        for event in events:
            self.publish(event)



class SubscriberNotRegisteredException(Exception):
    def __init__(self, subscriber_id: str):
        self.subscriber_id = subscriber_id
        super().__init__(f"Subscriber with id {subscriber_id} has not been registered and cannot be unregistered")
